#ifndef CRITERIA_H
#define CRITERIA_H

#if defined(__cplusplus)
extern "C" {
#endif

#include "cb.h"
#include "DistCrit/distance.h"

typedef enum { CR_User, CR_MSE, CR_MAE, CR_FTest,
    CR_StochasticComplexity, CR_MDL, CR_XieBeniKwon, CR_DBI } CR_TYPE;

/* Initialize function gets a training set that it may use to 
   calculate necessary data. It returns a void pointer that is in turn
   passed on to finalization function. */
typedef void* (*CR_InitializeFunction)(DistanceInfo*, TRAININGSET*);
typedef float (*CR_CriterionFunction)
    (DistanceInfo*, TRAININGSET*, CODEBOOK*, PARTITIONING*, void*);
typedef void (*CR_FinalizeFunction)(void*);
typedef int (*CR_PartitionFunction)(
    DistanceInfo*, TRAININGSET*, CODEBOOK*, PARTITIONING*, int*, void*);
typedef int (*CR_CentroidFunction)(
    DistanceInfo*, TRAININGSET*, CODEBOOK*, PARTITIONING*, int*, void*);
typedef int (*CR_LocalPartitionFunction)(
    DistanceInfo*, TRAININGSET*, CODEBOOK*, PARTITIONING*, int*, int, void*);

typedef struct {
    CR_TYPE Type;
    CR_CriterionFunction Func;
    CR_FinalizeFunction Done;
    CR_PartitionFunction Partition;
    CR_CentroidFunction Centroid;
    CR_LocalPartitionFunction Local;
    void* Data;
    TRAININGSET* TS;
    DistanceInfo* DI;
} CriterionInfo;

CriterionInfo* ciNew(TRAININGSET* TS, DistanceInfo* DI,
    CR_TYPE Criterion, CR_InitializeFunction Init,
    CR_CriterionFunction Func, CR_FinalizeFunction Done,
    CR_PartitionFunction Partition, CR_LocalPartitionFunction Local,
    CR_CentroidFunction Centroid);
void ciDelete(CriterionInfo* CI);
/*
float ciEvaluate(CriterionInfo* CI, CODEBOOK* CB, PARTITIONING* PA);
*/

#define ciEvaluate(CI, CB, PA) \
    ((CI)->Func)((CI)->DI, (CI)->TS, (CB), (PA), (CI)->Data)

#define ciPartitionOptimally(CI, CB, PA, ChangedCentroids) \
    ((CI)->Partition)((CI)->DI, (CI)->TS, (CB), (PA), (ChangedCentroids), (CI)->Data)
#define ciLocalPartition(CI, CB, PA, Changes, ChangedCentroid) \
    ((CI)->Local)((CI)->DI, (CI)->TS, (CB), (PA), (Changes), (ChangedCentroid), (CI)->Data)
#define ciCalculateOptimalCentroids(CI, CB, PA, ChangedCentroids) \
    ((CI)->Centroid)((CI)->DI, (CI)->TS, (CB), (PA), (ChangedCentroids), (CI)->Data)

#define ciCriterionDistance(CI) ((CI)->DI)

DT_TYPE ciCriterionDefaultDistance(CR_TYPE Criterion);

int FindNearestCluster(DistanceInfo* DI, TRAININGSET* TS,
    CODEBOOK* CB, PARTITIONING* PA, int ForVector, int InCluster, int* Changed);

int MSECentroid(DistanceInfo* DI, TRAININGSET* TS,
    CODEBOOK* CB, PARTITIONING* PA, int* Changed, void* UnusedParameter);

int MSEPartition(DistanceInfo* DI, TRAININGSET* TS,
    CODEBOOK* CB, PARTITIONING* PA, int* Changed, void* AuxiliaryData);

int MSELocal(DistanceInfo* DI, TRAININGSET* TS, CODEBOOK* CB,
    PARTITIONING* PA, int* Changes, int Changed, void* UnusedParameter);

#if defined(__cplusplus)
}
#endif

#endif

