/****************************************************************

    FTEST.C

    20.6.2001

    Ismo Krkkinen

    Implements F-test.

****************************************************************/

#include "DistCrit/ftest.h"
#include "owntypes.h"
#include "memctrl.h"
#include "DistCrit/criteria.h"
#include <string.h>
/*#include "sqmatrix.h"*/

static void MeanOfTS(TRAININGSET* TS, VECTORTYPE Result) {
    int n, k, f;
    llong* temp = allocate(sizeof(llong) * VectorSize(TS));
    memset(temp, 0, sizeof(llong) * VectorSize(TS));
    for (n = 0; n < BookSize(TS); n++) for (k = 0; k < VectorSize(TS); k++)
        for (f = 0; f < VectorFreq(TS, n); ++f) 
            temp[k] += VectorScalar(TS, n, k);
    for (k = 0; k < VectorSize(TS); k++) 
        Result[k] = (int)(temp[k] / TotalFreq(TS));
    deallocate(temp);
}


typedef struct {
    CODEBOOK Average;
    PARTITIONING NMSE_PA;
    CriterionInfo* CI;
    int* NewMap;
} FTestData;

void* InitFTest(DistanceInfo* DI, TRAININGSET* TS) {
    FTestData* D = allocate(sizeof(FTestData));
    CreateNewCodebook(&(D->Average), 1, TS);
    D->CI = ciNew(TS, DI, CR_MSE, 0, 0, 0, 0, 0, 0);
    MeanOfTS(TS, Vector(&(D->Average), 0));
    VectorFreq(&(D->Average), 0) = TotalFreq(TS);
    CreateNewPartitioning(&(D->NMSE_PA), TS, 1);
    D->NewMap = allocate(sizeof(int) * BookSize(TS));
    return (void*)D;
}

int FTestPartition(DistanceInfo* DI, TRAININGSET* TS,
    CODEBOOK* CB, PARTITIONING* PA, int* Changed, void* AuxiliaryData)
{
    FTestData* D = (FTestData*)AuxiliaryData;
    return MSEPartition(DI, TS, CB, PA, Changed, D->NewMap);
}

void DoneFTest(void* AuxiliaryData) {
    FTestData* D = (FTestData*)AuxiliaryData;
    /*diDelete(D->CI->DI);*/
    ciDelete(D->CI);
    FreeCodebook(&(D->Average));
    FreePartitioning(&(D->NMSE_PA));
    deallocate(D->NewMap);
    deallocate(AuxiliaryData);
}

/*
static double OldFTest(TRAININGSET* TS, CODEBOOK* CB, PARTITIONING* PA) {
    int k;
    llong QB = 0;
    double NMSE;
    VECTORELEMENT v[100];
    if ((BookSize(CB) <= 1) && (BookSize(TS) == BookSize(CB)))
        return 1000000000.0;
    MeanOfTS(TS, v);
    for (k = 0; k < BookSize(CB); k++) QB += VectorFreq(CB, k) 
        * VectorDist(v, Vector(CB, k), VectorSize(CB));
    QB /= VectorSize(TS);
    NMSE = BookSize(TS) * AverageErrorForSolution(TS, CB, PA, MSE);
    NMSE /= BookSize(TS) - BookSize(CB);
    return 1.0 / ((QB / (double)(BookSize(CB) - 1)) / NMSE);
}
*/

float CalculateFTest(DistanceInfo* DI, TRAININGSET* TS,
    CODEBOOK* CB, PARTITIONING* PA, void* AuxiliaryData)
{
    int k;
    float NMSE, QB;
    FTestData* D = (FTestData*)(AuxiliaryData);
    if ((BookSize(CB) <= 1) && (BookSize(TS) == BookSize(CB)))
	return 1000.0;
    QB = 0;
    for (k = 0; k < BookSize(CB); k++) QB += VectorFreq(CB, k) *
	diDistance(ciCriterionDistance(D->CI),
	    CB, &(D->Average), &(D->NMSE_PA), k, 0, DT_DATA2MODEL);
    QB /= VectorSize(TS) * (BookSize(CB) - 1);
    NMSE = TotalFreq(TS) * ciEvaluate(D->CI, CB, PA);
    NMSE /= BookSize(TS) - BookSize(CB);
    return NMSE / QB;
}

