#ifndef CB_UTIL_H
#define CB_UTIL_H

#if defined(__cplusplus)
extern "C" {
#endif

#include "cb.h"
#include "DistCrit/criteria.h"

void RemoveDuplicatesFromCodebook(CODEBOOK* CB);

void GenerateRandomVector(TRAININGSET* TS, CODEBOOK* CB, int Index);
void GenerateRandomCodebook(TRAININGSET* TS, CODEBOOK* CB);

TRAININGSET* CreateTrainingSetFromRawData(char* Filename, 
					  int FrequencyLocation,
					  double Minimum, double Maximum,
					  int Dimensionality, int Count,
					  int BytesPerElement, int HasHeader,
					  int QuietLevel);

int CreateRawFile(char* OrigName, char* InName, char* OutName, CODEBOOK* CB, 
		  int FrequencyInfo, double Minimum, double Maximum,
		  int Header);

enum { RGD_CB = 1, RGD_TXT = 2 };

int ReadGenericData(char* Filename, TRAININGSET* TS, int* Format,
    float* Minimum, float* Maximum);
int ReadGenericMapping(char* Filename, TRAININGSET* TS, PARTITIONING* PA,
    int* Format);

void WriteGenericData(char* Filename, CODEBOOK* CB, int Format,
    float Minimum, float Maximum, int AllowOverwrite);
void WriteGenericMapping(char* Filename, TRAININGSET* TS, PARTITIONING* PA,
    int Format, int AllowOverwrite);

void CopyCodebookOmitting(CODEBOOK* To, CODEBOOK* From, int Index);


int* FrequencyToIndexMap(TRAININGSET* TS);

#define SWAP_NONE 0
#define SWAP_RANDOM 1

int SelectNonCodevector(TRAININGSET* TS, CODEBOOK* CB, int* FreqToIndexMap);
int SwapCentroid(CriterionInfo* CI, TRAININGSET* TS,
    CODEBOOK* CB, PARTITIONING* PA, int SwapMethod,
    int* FromCluster, int* ToCluster, int* FreqToIndexMap);

void CheckCodebook(TRAININGSET* TS, CODEBOOK* CB);
void CheckPartitioning(TRAININGSET* TS, CODEBOOK* CB, PARTITIONING* PA);

void FixCodebookFrequencies(CODEBOOK* CB, PARTITIONING* PA);

#if defined(__cplusplus)
}
#endif

#endif

