/*
 * gcc -I. -ggdb test_matrix_inv.cpp -o test_matrix_inv -lm
 */

#include "matrix.hpp"
#include "matrix.cpp"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

int main(int argc, char** argv) {
    if (argc != 3) {
        fprintf(stderr, "Usage: test_matrix_inv points dim\n\n");
        return 1;
    }
    int N = atoi(argv[1]);
    int D = atoi(argv[2]);
    if (N <= 0 || D <= 0) {
        fprintf(stderr, "points or dim is <= 0.\n\n");
        return 2;
    }
    srand((unsigned int)getpid());
    Matrix C(D, D);
    C.Fill(0.0f);
    Matrix V(D, 1);
    Matrix OP(D, D);
    for (int n = 0; n < N; ++n) {
        V.Randomize();
        OP.OuterProduct(V, V);
        C += OP;
    }
    C *= 1.0f / N;
    Matrix I = C * C.Inverse();
    I.Print(stdout);
    float MaxDiff = 0.0f;
    for (int r = 0; r < D; ++r) for (int c = 0; c < D; ++c) if (r != c) {
        float AbsDiff = fabs(I(r, c));
        if (AbsDiff > MaxDiff) MaxDiff = AbsDiff;
    }
    printf("Determinant: %g, maximum absolute non-diagonal: %g\n\n",
        I.Determinant(), MaxDiff);
    return 0;
}

