#if !defined(FUZZY_HPP)
#define FUZZY_HPP

#if defined(__cplusplus)
extern "C" {
#endif

#include "cb.h"

#if defined(__cplusplus)
}
#endif

#include "matrix.hpp"
#include "DistCrit/distance.h"

#if defined(__cplusplus)

class FuzzyPartitioning {
private:
    float power;
    Matrix M;
    TRAININGSET* dataset;
public:
    // Add file I/O functions later.

    void Fuzzify(PARTITIONING* PA);

    FuzzyPartitioning() : power(2.0f), M(0, 0), dataset(0) { }
    FuzzyPartitioning(TRAININGSET* TS, int Groups, float FuzzyPower = 2.0f);
    FuzzyPartitioning(TRAININGSET* TS, CODEBOOK* CB, PARTITIONING* PA,
        float FuzzyPower = 2.0f);

    void UpdateMemberships(CODEBOOK* CB, DistanceInfo* DI);
    void Crispify(PARTITIONING* PA);

    inline int Vectors() const { return M.Rows(); }
    inline int Groups() const { return M.Cols(); }
    inline float operator()(int Vector, int Group) { return M(Vector, Group); }

    inline ConstRowIterator RowBegin(int Vector) const
        { return M.ConstRowBegin(Vector); }
    inline ConstRowIterator RowEnd(int Vector) const
        { return M.ConstRowEnd(Vector); }
};

#else

typedef struct {
    float FuzzyPower;
    MATRIX_TYPE M;
} FuzzyPartitioning;

#endif /* __cplusplus */

#if defined __cplusplus
extern "C" {
#endif

FuzzyPartitioning* fpNew(TRAININGSET* TS, int Groups, float FuzzyPower);
void fpDelete(FuzzyPartitioning* FPA);
void fpUpdateMemberships(FuzzyPartitioning* FPA, CODEBOOK* CB,
    DistanceInfo* DI);
float fpMembership(FuzzyPartitioning* FPA, int Vector, int Group);

int fpVectors(FuzzyPartitioning* FPA);
int fpGroups(FuzzyPartitioning* FPA);

FuzzyPartitioning* Fuzzify(TRAININGSET* TS, PARTITIONING* PA,
    FuzzyPartitioning* FPA);

void Crispify(FuzzyPartitioning* FPA, PARTITIONING* PA);

#if defined(__cplusplus)
}
#endif

#endif

